/*
 * Copyright (c) 2013, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 */
/*
 * This file is licensed according to the BSD 3-clause license as follows:
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the "Linz Center of Mechatronics GmbH" and "LCM" nor
 *       the names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL "Linz Center of Mechatronics GmbH" BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * This file is part of X2C. http://x2c.lcm.at/
 * $LastChangedRevision: 1603 $
 */
/* USERCODE-BEGIN:Description                                                                                         */
/* Description: */
/* USERCODE-END:Description                                                                                           */
#include "ConvFnc_Average_FiP16.h"

/* USERCODE-BEGIN:PreProcessor                                                                                        */
/* USERCODE-END:PreProcessor                                                                                          */

extern uint8 MaskParamBuffer[];
extern uint8 ImplParamBuffer[];
extern uint8 SaveFncDataBuffer[];

/** Implementation parameter data structure */
typedef struct {
    uint16 n;
    uint8 sfrn;
} AVERAGE_FIP16_IMPL_PARAM;

/** n enumeration */
typedef enum {
    N_1 = 0,
    N_2 = 1,
    N_4 = 2,
    N_8 = 3,
    N_16 = 4,
    N_32 = 5,
    N_64 = 6,
    N_128 = 7,
    N_256 = 8
} tn;

/** Private prototypes */
static uint8 convertM2I(const AVERAGE_FIP16_MASK_PARAM *maskParam, AVERAGE_FIP16_IMPL_PARAM *impParam, float_CoT ts);

/**
 * @brief Converts Mask parameters to Implementation parameters.
 *
 * @param[in] maskParam Mask parameters
 * @param[out] impParam Implementation parameters
 * @param[in] ts Block sample time
 *
 * @return Error: zero on success, not zero in case of conversion error
 */
static uint8 convertM2I(const AVERAGE_FIP16_MASK_PARAM *maskParam, AVERAGE_FIP16_IMPL_PARAM *impParam, float_CoT ts)
{
    uint8 error = (uint8)0;
/* USERCODE-BEGIN:Conversion                                                                                          */
    switch(maskParam->n)
    {
        case N_1:
            impParam->n = 1;
            impParam->sfrn = 0;
            break;
        case N_2:
            impParam->n = 2;
            impParam->sfrn = 1;
            break;
        case N_4:
            impParam->n = 4;
            impParam->sfrn = 2;
            break;
        case N_8:
            impParam->n = 8;
            impParam->sfrn = 3;
            break;
        case N_16:
            impParam->n = 16;
            impParam->sfrn = 4;
            break;
        case N_32:
            impParam->n = 32;
            impParam->sfrn = 5;
            break;
        case N_64:
            impParam->n = 64;
            impParam->sfrn = 6;
            break;
        case N_128:
            impParam->n = 128;
            impParam->sfrn = 7;
            break;
        case N_256:
            impParam->n = 256;
            impParam->sfrn = 8;
            break;
        default:
            impParam->n = 1;
            impParam->sfrn = 0;
            break;
    }

/* USERCODE-END:Conversion                                                                                            */
    return (error);
}

/**
 * @brief Load block mask parameter data.
 *
 * @param[in] maskParam Mask parameter data structure
 * @param[out] data Data
 * @param[out] dataLen Data length
 * @param[in] maxSize Maximum Service data length
 *
 * @return Error: zero on success, not zero in case of load- or conversion error
 */
uint8 Average_FiP16_LoadMP(const AVERAGE_FIP16_MASK_PARAM *maskParam, uint8 data[], uint16* dataLen, uint16 maxSize)
{
    uint8 error = (uint8)0;
    if ((uint16)9 > maxSize)
    {
        error = (uint8)1;
    }
    else
    {
        data[0] = (uint8)(maskParam->n);
        data[1] = (uint8)((*(uint64*)&(maskParam->ts_fact)) & 0x00000000000000FF);
        data[2] = (uint8)((*(uint64*)&(maskParam->ts_fact) >> 8) & 0x00000000000000FF);
        data[3] = (uint8)((*(uint64*)&(maskParam->ts_fact) >> 16) & 0x00000000000000FF);
        data[4] = (uint8)((*(uint64*)&(maskParam->ts_fact) >> 24) & 0x00000000000000FF);
        data[5] = (uint8)((*(uint64*)&(maskParam->ts_fact) >> 32) & 0x00000000000000FF);
        data[6] = (uint8)((*(uint64*)&(maskParam->ts_fact) >> 40) & 0x00000000000000FF);
        data[7] = (uint8)((*(uint64*)&(maskParam->ts_fact) >> 48) & 0x00000000000000FF);
        data[8] = (uint8)((*(uint64*)&(maskParam->ts_fact) >> 56) & 0x00000000000000FF);
        *dataLen = (uint16)9;
    }
    return (error);
}

/**
 * @brief Save block mask parameter data.
 *
 * @param[in] block Pointer to block structure
 * @param[out] maskParam Mask parameter data structure
 * @param[in] data Data
 * @param dataLen Length of mask parameter data stream
 *
 * @return Error: zero on success, not zero in case of save- or conversion error
 */
uint8 Average_FiP16_SaveMP(AVERAGE_FIP16 *block, AVERAGE_FIP16_MASK_PARAM *maskParam, const uint8 data[], uint16 dataLen)
{
    uint8 error = (uint8)0;

    if (dataLen != (uint16)9)
    {
        error = (uint8)1;
    }
    else
    {
        uint64 tmp64;
        float_CoT block_ts;
        AVERAGE_FIP16_IMPL_PARAM *implParam = (AVERAGE_FIP16_IMPL_PARAM*)ImplParamBuffer;

        /** - Cache old mask parameters */
        memcpy(MaskParamBuffer, maskParam, sizeof(*maskParam));

        /** - Save new mask parameters */
        maskParam->n = data[0];
        tmp64 = (uint64)data[1] + \
            ((uint64)data[2] << 8) + ((uint64)data[3] << 16) + \
            ((uint64)data[4] << 24) + ((uint64)data[5] << 32) + \
            ((uint64)data[6] << 40) + ((uint64)data[7] << 48) + \
            ((uint64)data[8] << 56);
        maskParam->ts_fact = (float64)(*(float64*)&tmp64);

        /** - Get Block Sample Time */
        block_ts = maskParam->ts_fact * getModelSampleTime();
        /** - Convert mask parameters to implementation parameters */
        error = convertM2I(maskParam, implParam, block_ts);

        /** - Prepare implementation parameters for save function */
        SaveFncDataBuffer[0] = (uint8)(implParam->n & 0x00FF);
        SaveFncDataBuffer[1] = (uint8)((implParam->n >> 8) & 0x00FF);
        SaveFncDataBuffer[2] = (uint8)implParam->sfrn;

        /** - Execute save function of block */
        error = Average_FiP16_Save(block, SaveFncDataBuffer, 3);

        /** - Check for errors during execution of save function */
        if (error != 0)
        {
            /* Restore old mask parameter */
            memcpy(maskParam, MaskParamBuffer, sizeof(*maskParam));
        }

    }
    return (error);
}

/**
 * @brief Initializes Implementation parameters from Mask parameters.
 *
 * @param[out] block Block
 * @param[in] maskParam Mask parameters
 *
 * @return Error: zero on success, not zero in case of initialization error
 */
uint8 Average_FiP16_InitMP(AVERAGE_FIP16 *block, const AVERAGE_FIP16_MASK_PARAM *maskParam)
{
    uint8 error = (uint8)0;
    float_CoT block_ts;

    AVERAGE_FIP16_IMPL_PARAM *implParam = (AVERAGE_FIP16_IMPL_PARAM*)ImplParamBuffer;

    /** - Get Block Sample Time */
    block_ts = maskParam->ts_fact * getModelSampleTime();
    /** - Convert mask parameters to implementation parameters */
    error = convertM2I(maskParam, implParam, block_ts);

    /** - Prepare implementation parameters for save function */
    SaveFncDataBuffer[0] = (uint8)(implParam->n & 0x00FF);
    SaveFncDataBuffer[1] = (uint8)((implParam->n >> 8) & 0x00FF);
    SaveFncDataBuffer[2] = (uint8)implParam->sfrn;

    /** - Execute save function of block */
    error = Average_FiP16_Save(block, SaveFncDataBuffer, 3);

    return (error);
}
