/**
 * 
 */
package de.ixxat.vci3;

import de.ixxat.vci3.bal.IBalObject;

/**
 * This interface represents a VCI device object. When no longer needed the VCI
 * device object has to be disposed using the {@link IDisposable} interface. Get
 * a VCI device object via the {@link IVciDeviceManager#OpenDevice(long)}.
 * 
 * @author Michael Ummenhofer
 * 
 */
public interface IVciDevice extends IDisposable
{
  /**
   * Gets information to the VCI device.
   * 
   * @return {@link VciDeviceInfo} - VCI Device Information. Information to the VCI
   *         device.
   * @throws Throwable
   */
  public VciDeviceInfo GetDeviceInfo() throws Throwable;

  /**
   * Gets the capabilities of the VCI device.
   * 
   * @return {@link VciDeviceCapabilities} - VCI Device Capabilities. Capabilities of the VCI device.
   * @throws Throwable
   */
  public VciDeviceCapabilities GetDeviceCaps() throws Throwable;
  
  /**
   * This method is called to open the Bus Access Layer.
   * 
   * @return {@link IBalObject} - Bus Access Layer Object.
   * @throws Throwable
   */
  public IBalObject OpenBusAccessLayer() throws Throwable;

}
