/**
 * 
 */
package de.ixxat.vci3;

import de.ixxat.jni.CFunction;
import de.ixxat.jni.CMalloc;

/**
 * This class implements the basic VCI exception object.
 * 
 * @author Michael Ummenhofer
 * 
 */
public class VciException extends Exception
{
  /** For serialization */ 
  static final long serialVersionUID = 1;
  
  /**
   * maximum length of an error string
   */ 
  private static final int VCI_MAX_ERRSTRLEN = 256;
  
  // DLL exported functions
  private static final CFunction m_fpVciFormatError;
  
  // Initialization of static members
  static
  {
    m_fpVciFormatError = new CFunction("vciapi", "VciFormatError", "JNI");
  }

  /**
   * HRESULT error code returned from native VCI function
   */
  public int m_hResult = 0;
  
  /**
   * Constructs an new VCI exception object.
   * 
   * @param strErrorMessage
   *          The basic error message.
   * @param hResult
   *          HRESULT error code returned from native VCI function.
   */
  public VciException(String strErrorMessage, int hResult)
  {
    super(strErrorMessage);
    
    // Store HRESULT code
    m_hResult = hResult;
  }
  
  /**
   * Converts the error code in an human readable error message text.
   * 
   * @return The converted human readable error message.
   */
  public String VciFormatError()
  {
    int     hResult   = 0;
    String  strErrMsg = "n/a";

    // Get formatted error string
    CMalloc pszError = new CMalloc(VCI_MAX_ERRSTRLEN);

    try
    {
      // Try to get error message
      hResult = m_fpVciFormatError.callInt(new Object[]{ new Integer(m_hResult), pszError });
      if(hResult == 0)
      {
        strErrMsg = pszError.getString(0);
      }
    }
    catch(Throwable oThrowable)
    {
      
    }
    finally
    {
      // Free memory
      pszError.free();
    }
    
    return strErrMsg;
  }
 
}
