/**
 * 
 */
package de.ixxat.vci3.bal;

import de.ixxat.vci3.IDisposable;

/**
 * This interface represents a BAL (Bus Access Layer) object. When no longer
 * needed the BAL object has to be disposed using the {@link IDisposable}
 * interface. Get the BAL object of a device by using method
 * {@link de.ixxat.vci3.IVciDevice#OpenBusAccessLayer()}.
 * 
 * @author Michael Ummenhofer
 * 
 */
public interface IBalObject extends IDisposable
{
  /**
   * Gets the features of the BAL Object.
   * 
   * @return {@link BalFeatures} - BalFeatures is used to get the Features of
   *         the BAL Object.
   * @throws Throwable
   */
  public BalFeatures GetFeatures() throws Throwable;
  
  /**
   * This method opens the specified bus socket.
   * 
   * @param dwPortNumber
   *          Number of the bus socket to open. This parameter must be within
   *          the range of 0 to max port number - 1.
   * @param dwSocketType
   *          Type of the bus socket to open. The supported socket types are
   *          depending on the BusType of the BAL resource specified by the
   *          dwPortNumber parameter. I.e. for a CAN bus socket the following
   *          socketTypes are supported: <blockquote>
   *          <li>{@link de.ixxat.vci3.bal.can.ICanSocket} use type
   *          {@link IBalResource#IID_ICanSocket}</li>
   *          <li>{@link de.ixxat.vci3.bal.can.ICanControl} use type
   *          {@link IBalResource#IID_ICanControl}</li>
   *          <li>{@link de.ixxat.vci3.bal.can.ICanScheduler} use type
   *          {@link IBalResource#IID_ICanScheduler}</li>
   *          </blockquote> It's possible have several socketType open at the
   *          same time (i.e. {@link de.ixxat.vci3.bal.can.ICanControl} and
   *          {@link de.ixxat.vci3.bal.can.ICanSocket}).
   * 
   * @return {@link IBalResource} - If the method succeeds it returns the opened
   *         bus socket object as {@link IBalResource} reference. This reference
   *         can be casted to the type specified by parameter dwSocketType. If
   *         the method fails it throws an exception. When no longer needed the
   *         returned socket object has to be disposed using the
   *         {@link IDisposable} interface.
   * @throws Throwable
   */
  public IBalResource OpenSocket(int dwPortNumber, int dwSocketType) throws Throwable;

}
